# test_bodhi_update_state.py
#  - Test class to test bodhi_update_state in AutoQA
#
# Copyright 2011, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author:
#       Tim Flink <tflink@redhat.com>

import datetime

from autoqa import bodhi_update_state

# utility function to change string timestamp into a datetime
def get_time(timestr):
    return datetime.datetime.strptime(timestr, '%Y-%m-%d %H:%M:%S')

# utility function to create a result object as used by bodhi_update_status
def make_result(time, testname, result, arch='noarch'):
    return {'time':time, 'testname':testname, 'result':result, 'arch':arch}

# tests for bodhi_update_state
class TestBodhiUpdateState:

    ref_fail_results = [
        make_result(get_time('2011-05-14 09:00:21'),
                                'depcheck', result='FAILED', arch='i386'),
        make_result(get_time('2011-05-14 09:00:21'),
                                'depcheck', result='FAILED', arch='x86_64'),
        make_result(get_time('2011-05-14 09:00:21'),
                                'upgradepath', result='FAILED', arch='noarch')
    ]

    ref_update_name = 'update-1.2.3'

    # setup to be run before every test method
    def setup_method(self, method):
        self.test_state = bodhi_update_state.BodhiUpdateState(self.ref_update_name)


    def test_add_test_to_state(self, monkeypatch):
        ref_result = make_result(get_time('2011-05-14 09:00:21'),
                            'depcheck', result='PASSED',arch='x86_64')

        self.test_state.add_result(ref_result)

        assert self.test_state.get_state() == bodhi_update_state.INCOMPLETE


    def should_be_pass_minimum_tests(self, monkeypatch) :
        self.test_state.test_states['depcheck_32'] = 'PASSED'
        self.test_state.test_states['depcheck_64'] = 'PASSED'
        self.test_state.test_states['upgradepath'] = 'PASSED'

        assert self.test_state.get_state() is bodhi_update_state.PASS


    def should_be_incomplete_2_depcheck_no_upgradepath(self, monkeypatch) :
        self.test_state.test_states['depcheck_32'] = 'PASSED'
        self.test_state.test_states['depcheck_64'] = 'PASSED'

        assert self.test_state.get_state() is bodhi_update_state.INCOMPLETE


    def should_be_incomplete_missing_single_depcheck(self, monkeypatch):
        self.test_state.test_states['depcheck_32'] = 'PASSED'
        self.test_state.test_states['upgradepath'] = 'PASSED'

        assert self.test_state.get_state() is bodhi_update_state.INCOMPLETE


    def should_be_fail_minimum_tests_1_fail(self, monkeypatch):
        self.test_state.test_states['depcheck_32'] = 'FAILED'
        self.test_state.test_states['depcheck_64'] = 'PASSED'
        self.test_state.test_states['upgradepath'] = 'PASSED'

        assert self.test_state.get_state() is bodhi_update_state.FAIL


    def should_be_pass_minimum_1fail_topass(self, monkeypatch):
        self.test_state.test_states['depcheck_32'] = 'PASSED'
        self.test_state.test_states['depcheck_64'] = 'PASSED'
        self.test_state.test_states['upgradepath'] = 'FAILED'

        ref_result = make_result(get_time('2011-05-14 09:00:21'),
                            'upgradepath', result='PASSED', arch='noarch')

        self.test_state.add_result(ref_result)

        assert self.test_state.get_state() is bodhi_update_state.PASS


    def should_be_fail_minimum_pass_1tofail(self, monkeypatch):
        self.test_state.test_states['depcheck_32'] = 'PASSED'
        self.test_state.test_states['depcheck_64'] = 'PASSED'
        self.test_state.test_states['upgradepath'] = 'PASSED'

        ref_result = make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                                        result='PASSED', arch='x86_64')

        self.test_state.add_result(ref_result)

        assert self.test_state.get_state() is bodhi_update_state.PASS


    def should_pass_updates_testing_just_depcheck(self):
        # interested in a non-default tag here, recreate object
        self.test_state = bodhi_update_state.BodhiUpdateState(self.ref_update_name,
                    kojitag='updates-testing')

        self.test_state.test_states['depcheck_32'] = 'PASSED'

        ref_result = make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                                        result='PASSED', arch='x86_64')
        self.test_state.add_result(ref_result)

        assert self.test_state.get_state() is bodhi_update_state.PASS


    # the idea here is that we want to put a whole bunch of failed results
    # into the state object and make sure that they start passing
    # this should probably be moved into test_bodhi_utils
    def should_be_pass_manyfails_to_allpass(self, monkeypatch):
        ref_num_failruns = 5

        ref_pass_results = [
            make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='PASSED', arch='i386'),
            make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='PASSED', arch='x86_64'),
            make_result(get_time('2011-05-14 09:00:21'),
                                    'upgradepath', result='PASSED', arch='noarch')
        ]

        # let the failing begin
        for run in range(0, ref_num_failruns):
            for result in self.ref_fail_results:
                self.test_state.add_result(result)

        # now magically fix it
        for result in ref_pass_results:
            self.test_state.add_result(result)

        assert self.test_state.get_state() is bodhi_update_state.PASS


    def should_keep_result_history_correct_num(self, monkeypatch) :
        ref_num_failruns = 5

        # let the failing begin
        for run in range(0, ref_num_failruns):
            for result in self.ref_fail_results:
                self.test_state.add_result(result)

        assert self.test_state.num_results() == ref_num_failruns * len(self.ref_fail_results)


    def should_keep_result_history_content(self, monkeypatch):
        ref_result = make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='PASSED', arch='x86_64')

        self.test_state.add_result(ref_result)

        assert self.test_state.result_history[0] is ref_result

    def should_detect_test_state_change(self):
        ref_result = make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='PASSED', arch='x86_64')
        for result in self.ref_fail_results:
            self.test_state.add_result(result)

        self.test_state.add_result(ref_result)

        assert self.test_state.did_test_change()

    def should_detect_test_state_no_change(self):
        ref_result = make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='FAILED', arch='x86_64')
        for result in self.ref_fail_results:
            self.test_state.add_result(result)

        self.test_state.add_result(ref_result)

        assert not self.test_state.did_test_change()

    def should_accept_i686_as_32bit(self):
        base_results = [
            make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='PASSED', arch='x86_64'),
            make_result(get_time('2011-05-14 09:00:21'),
                                    'upgradepath', result='PASSED', arch='noarch')
        ]

        ref_result = make_result(get_time('2011-05-14 09:00:21'),
                                    'depcheck', result='PASSED', arch='i686')

        for result in base_results:
            self.test_state.add_result(result)

        self.test_state.add_result(ref_result)

        assert self.test_state.get_state() is bodhi_update_state.PASS
