#
# test_bodhi_utils.py - unit tests for bodhi_utils
#
# Copyright 2011, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author:
#       Tim Flink <tflink@redhat.com>
import datetime

import bunch

from autoqa import bodhi_utils
from autoqa import config


default_logurl = 'http://localhost/logs/output.log'

# utility methods for testing
def get_time(timestr):
    return datetime.datetime.strptime(timestr, '%Y-%m-%d %H:%M:%S')

def make_result(time, testname, result, arch='noarch'):
    return {'time':time, 'testname':testname, 'result':result, 'arch':arch}

def make_comment(time, karma, username, comment):
    return bunch.Bunch({'anonymous':'False', 'author':username, 'group':'None',
                        'karma':str(karma), 'text':comment,
                        'timestamp':str(time)})

def make_autoqa_comment(time, testname, result, arch='noarch',
                        logurl=default_logurl):
    comment = 'AutoQA: %s test %s on %s. Result log: %s (results are'\
              ' informative only)' % (testname, result, arch, logurl)

    return make_comment(time, 0, 'autoqa', comment)

# This just stubs out the config object so that it's easier to change the
# settings around
class StubConfig():
    def __init__(self, config):
        self.config = config

    # check the config dict for the setting instead of looking for the actual
    # attr
    def __getattr__(self, name):
        return self.config[name]

# tests for _is_bodhi_comment_email_needed in bodhi_utils
class TestIsEmailNeeded:

    default_kojitag = 'updates'

    default_email_settings = {'email_always_send': False,
                              'email_test_state_change': False,
                              'email_updates_testing': True,
                              'email_timed_retest': True,
                              'email_all_passed': False,
                              'email_update_state_change':True
                             }

    email_settings = {}

    # this is a py.test specific function that generates an arg for anything
    # named bodhitest. The point of it is to do dependency injection on the 
    # configuration settings so that we don't have to duplicate the
    # monkeypatching code every time
    def pytest_funcarg__bodhiemail(self, request):
        self.config_stub = StubConfig(self.email_settings)
        mp = request.getfuncargvalue("monkeypatch")
        mp.setattr(config.Config, "bodhi_email", self.config_stub)
        return mp

    # setup to be run before every test
    def setup_method(self, method):
        self.new_result = make_result(get_time('2011-05-14 09:30:00'),
                                     'depcheck', result='PASSED', arch='x86_64')
        self.update_name = 'update-1.2.3'
        self.email_settings = self.default_email_settings.copy()


    def should_not_email_one_test(self, bodhiemail):
        ref_comments = []
        new_result = make_result(get_time('2011-05-14 09:00:21'),
                                 'depcheck', result='PASSED',arch='x86_64')


        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == False


    def should_email_all_tests_passed_configured(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_all_passed'] = True

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='PASSED', arch='i386')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                        'upgradepath', result='PASSED', arch='noarch')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                                ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == True


    def should_email_one_test_failed(self, bodhiemail):
        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='FAILED', arch='i386'),
        ]
        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                 'upgradepath', result='PASSED', arch='noarch')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                                ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == True


    def should_not_email_missing_upgradepath(self, bodhiemail):
        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='FAILED', arch='i386'),
        ]

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, self.new_result, kojitag=self.default_kojitag)

        assert test_email == False


    def should_not_email_missing_depcheck_arch(self, bodhiemail):
        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='FAILED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='PASSED', arch='x86_64'),
            make_result(get_time('2011-05-14 09:02:21'), 'upgradepath',
                        result='PASSED', arch='noarch')
        ]

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, self.new_result, kojitag=self.default_kojitag)

        assert test_email == False


    def should_not_send_all_pass_configured(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_all_passed'] = False

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='PASSED', arch='i386'),
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                 'upgradepath', result='PASSED', arch='noarch')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == False


    def should_email_update_pass_to_fail(self, bodhiemail):
        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='PASSED', arch='i386'),
            make_result(get_time('2011-05-14 09:02:21'), 'upgradepath',
                        result='PASSED', arch='noarch')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                        'upgradepath', result='FAILED', arch='noarch')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == True


    def should_email_update_fail_to_pass(self, bodhiemail):
        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='PASSED', arch='i386'),
            make_result(get_time('2011-05-14 09:02:21'), 'upgradepath',
                        result='FAILED', arch='noarch')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                        'upgradepath', result='PASSED', arch='noarch')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == True


    def should_not_email_update_pass_to_pass(self, bodhiemail):
        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='PASSED', arch='x86_64'),
            make_result(get_time('2011-05-14 09:02:21'), 'upgradepath',
                        result='PASSED', arch='noarch')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                 'upgradepath', result='FAILED', arch='noarch')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == False


    def should_send_updates_testing_configured(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_updates_testing'] = True

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED',arch='x86_64')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                 'depcheck', result='FAILED', arch='i386')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag='updates-testing')

        assert test_email == True


    def should_not_send_updates_testing_configured(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_updates_testing'] = False

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='PASSED', arch='x86_64')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                 'depcheck', result='FAILED', arch='i386')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag='updates-testing')

        assert test_email == False


    def should_not_email_test_fail_to_pass_configured(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_test_state_change'] = False

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='FAILED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='FAILED', arch='i386'),
            make_result(get_time('2011-05-14 09:02:21'), 'upgradepath',
                        result='FAILED', arch='noarch')
        ]

        new_result = make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                                 result='PASSED', arch='x86_64')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == False


    def should_email_test_fail_to_pass_configured(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_test_state_change'] = True

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='FAILED',arch='x86_64'),
            make_result(get_time('2011-05-14 09:01:21'), 'depcheck',
                        result='FAILED', arch='i386'),
            make_result(get_time('2011-05-14 09:02:21'), 'upgradepath',
                        result='FAILED', arch='noarch')
        ]

        new_result = make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                                 result='PASSED', arch='x86_64')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag=self.default_kojitag)

        assert test_email == True

    def should_not_override_updates_testing_setting(self, bodhiemail):
        # change email configuration from default
        self.email_settings['email_updates_testing'] = False
        self.email_settings['email_test_state_change'] = True

        ref_comments = [
            make_result(get_time('2011-05-14 09:00:21'), 'depcheck',
                        result='FAILED', arch='x86_64')
        ]

        new_result = make_result(get_time('2011-05-14 09:02:21'),
                                 'depcheck', result='PASSED', arch='x86_64')

        test_email = bodhi_utils._is_bodhi_comment_email_needed(self.update_name,
                            ref_comments, new_result, kojitag='updates-testing')

        assert test_email == False



# tests for _parse_result_from_comment in bodhi_utils
class TestParseBodhiComments:
    ref_first_time = get_time('2011-05-14 09:00:21')
    def should_parse_autoqa_pass(self):
        ref_comment = make_autoqa_comment(self.ref_first_time, 'depcheck',
                                          'PASSED', arch='x86_64')
        ref_result = make_result(self.ref_first_time, 'depcheck',
                                 result='PASSED', arch='x86_64')

        test_result = bodhi_utils._parse_result_from_comment(ref_comment)

        assert ref_result == test_result

