#!/usr/bin/env python
'''autotest_results_cleanup

Clean up the file tree with results from autotest.
Delete all files listed in USELESS_FILES_TEMPLATE and compress all large files.

Created for AutoQA purposes.

@author Jan Sedlak, Red Hat, 2012
@license GPLv3+
'''

import os
import subprocess
import re
import time
import logging
import pickle
from optparse import OptionParser

LAST_RUN_PATH = "/tmp/autoqa_clean.lastrun"
LOG_PATH = "/var/log/autoqa/autoqa_clean.log"

class CleanupError(Exception):
    '''Error occured during cleaning test results tree.'''
    pass

class TooYoungError(Exception):
    '''Desired test is too new, abort.'''
    pass

class NoTestDirError(Exception):
    '''Test doesn't include "status" file. Either it's not test directory
    or it's too new.'''
    pass

# list of files that are useless and should be deleted
USELESS_FILES_TEMPLATE = [
    ".parser_execute",
    ".archiver_execute",
    "<name>/debug/<name>.INFO",
    "<name>/debug/<name>.ERROR",
    "<name>/debug/<name>.WARNING",
    "<name>/sysinfo/df",
    "<name>/sysinfo/messages.gz",
    "<name>/sysinfo/iteration.1/meminfo.after",
    "<name>/sysinfo/iteration.1/slabinfo.after",
    "<name>/sysinfo/iteration.1/interrupts.before",
    "<name>/sysinfo/iteration.1/interrupts.after",
    "<name>/sysinfo/iteration.1/meminfo.before",
    "<name>/sysinfo/iteration.1/schedstat.before",
    "<name>/sysinfo/iteration.1/slabinfo.before",
    "<name>/sysinfo/iteration.1/schedstat.after",
    "<name>/sysinfo/reboot_current",
    "<name>/sysinfo/removed_packages",
    "<name>/sysinfo/added_packages",
    "<name>/sysinfo/dmesg.gz",
    ".archiving.log",
    ".parse.lock",
    "control.archive",
    ".autoserv_execute",
    ".parse.log",
    "control.srv",
    "job_report.html",
    "status",
    "sysinfo/modules",
    "sysinfo/version",
    "sysinfo/lspci_-vvn",
    "sysinfo/slabinfo",
    "sysinfo/partitions",
    "sysinfo/df",
    "sysinfo/uptime",
    "sysinfo/cmdline",
    "sysinfo/ld_--version",
    "sysinfo/gcc_--version",
    "sysinfo/proc_mounts",
    "sysinfo/hostname",
    "sysinfo/meminfo",
    "sysinfo/mount",
    "sysinfo/interrupts",
    "sysinfo/cpuinfo",
    "sysinfo/dmesg.gz",
    ]

def parse_arguments():
    '''parses arguments from sys.argv
    '''
    usage = "usage: %prog [-h] -d DIRECTORY [-s SIZE] [-a AGE] [-n]"
    parser = OptionParser(usage=usage)
    parser.add_option("-d", "--directory",
                      help="specify the directory to run this script on")
    parser.add_option("-s", "--size", default=10, type="float",
                      help="compress files larger than SIZE (in MB) "
                      "[default: %default]")
    parser.add_option("-a", "--age", default=60, type="int",
                      help="remove files only if the result is older than AGE "
                      "(in minutes) [default: %default]")
    parser.add_option("-n", "--dry-run", default=False, action="store_true",
                      help="don't remove anything, just display what would be done "
                      "[default: %default]")
    (options, args) = parser.parse_args()
    if not options.directory:
        parser.error('argument -d/--directory is required')
    if len(args) != 0:
        parser.error('unknown arguments %s' % ", ".join(args))
    return options

def load_last_run(path=LAST_RUN_PATH):
    '''load highest test number from last run of this program

    @param path path to lastrun file
    @return None if anything fails during reading, else latest number of test
    '''
    if not os.path.exists(path):
        return None
    
    try:
        fl = open(path, "r")
        highest_num = pickle.load(fl)
        fl.close()
        return highest_num
    except (pickle.PickleError, IOError):
        logging.info("can't load lastrun file %s", path, exc_info=True)
        return None

def save_last_run(testnum, path=LAST_RUN_PATH):
    '''save latest runned test number

    @param testnum number of latest runned test
    @param path path to lastrun file
    '''
    try:
        fl = open(path, "w")
        pickle.dump(testnum, fl)
        fl.close()
    except (pickle.PickleError, IOError):
        logging.error("can't write to lastrun file %s", path, exc_info=True)

def get_dirnames(path, testsdir):
    '''returns path to directory with log files of one test
    and relative path from root to this path

    @param path
    @param testdir path to root of tests
    @return resultdir, relative_dir
    '''

    index = path.find("/", len(testsdir)+1) # index of first '/' after testdir path
    if index != -1 and index != len(path) - 1: # if there is one
        rootdir = path[:index] # this is path to dir "XY-test", where are dirs
                               # with machines
        index = path.find("/", len(rootdir) + 1)
        if index != -1:
            resultdir = path[:index] # path to test directory itself
            relative_dir = path[index+1:] # the rest is relative path
        else:
            resultdir = path
            relative_dir = ''
    else:
        resultdir = None
        relative_dir = ''

    return resultdir, relative_dir

def get_dirs(path):
    '''returns all directories in directory 'path'

    @param path path to list directories from
    @return list of tuples (absolute_path, dirname)
    '''
    dirs = os.listdir(path)
    # get all directories, excluding symlinks to directories
    dirs = [(os.path.join(path, dr), dr) for dr in dirs
            if not os.path.isfile(os.path.join(path, dr)) and not
            os.path.islink(os.path.join(path, dr))]
    return dirs

def get_files(path):
    '''returns all files in directory 'path'

    @param path path to list files from
    @return list of tuples (absolute_path, dirname)
    '''
    files = os.listdir(path)
    # get all files, including symlinks
    files = [(os.path.join(path, fl), fl) for fl in files
             if os.path.isfile(os.path.join(path, fl)) or
             os.path.islink(os.path.join(path, fl))]
    return files

def get_test_name(path):
    '''tries to guess name of test in specified path.
    Raises exception NoTestDirError, if test name cannot be guessed.

    @param path path to guess test name in
    @return test name
    '''
    dirs = get_dirs(path)
    for dirpath, dr in dirs:
        # status file should be in <testname> directory
        if 'status' in os.listdir(dirpath):
            return dr
    raise NoTestDirError, "status file doesn't exist in result directory %s" % path

def check_time(path, difference):
    '''checks if the file is old enough to operate with

    @param path path to file to check
    @param difference how old the file should be (in minutes)
    @return True if the file is old enough, else False
    '''
    if os.path.islink(path):
        # don't check symlinks - if symlink points to
        # directory from which it removed files just before,
        # it shows "bad" modification time
        return True
    if time.time() - os.path.getmtime(path) > difference*60:
        return True
    else:
        return False

def check_dir_time(path, difference):
    '''check whole directory if all files are old enough

    @param path path to directory to check
    @param difference how old files should be (in minutes)
    @return True if all files is old enough, else False
    '''
    for dirpath, dirnames, filenames in os.walk(path):
        for filename in filenames: # for all files
            if not check_time(os.path.join(dirpath, filename), difference):
                return False
    return True

def remove_empty_dirs(path):
    '''recursively remove all empty directories or directories that
    contain only other empty directories.

    @param path to remove empty directories from
    '''
    for dirpath, dirnames, filenames in os.walk(path):
        try:
            os.removedirs(dirpath)
        except OSError: # directory isn't empty
            pass

def clean_files(args, path, useless_files):
    '''clean all unnecessary files from given path

    @param args Value instance of parsed args (sys.argv)
    @param path path to directory to clean
    @param useless_files list of files to remove
    '''
    files_to_remove = []

    # get path to dir with result from test and relative path from it
    resultdir, relativedir = get_dirnames(path, args.directory)

    if resultdir is None: # not in test result directory
        return

    files = get_files(path)
    for filepath, filename in files:
        # check if any file is too new
        if not check_time(filepath, args.age):
            raise TooYoungError, filepath
        if os.path.join(relativedir, filename) in useless_files:
            files_to_remove.append(filepath)

    if not files_to_remove:
        return

    if args.dry_run:
        print "rm %s" % " ".join(files_to_remove)
    else:
        for filename in files_to_remove:
            os.remove(filename) # remove them all

def compress_big_files(args, path):
    '''compress all big files in given directory

    @param args Value instance of parsed arguments
    @param path path to compress files in
    '''
    files_to_compress = []

    files = get_files(path)

    for filepath, filename in files:
        # check if file is bigger than desired size (convert it to bytes)
        if os.path.getsize(filepath) > args.size*1024*1024:
            # if it's not already compressed (and it's not symlink)
            if not os.path.islink(filepath) and filepath[-3:] != ".gz":
                files_to_compress.append(filepath)

    if not files_to_compress:
        return

    if args.dry_run:
        for filename in files_to_compress:
            print "gzip %s" % filename
            print "ln -s %s %s" % (os.path.basename(filename) + ".gz", filename)
    else:
        for filename in files_to_compress:
            try:
                # gzip it
                if subprocess.call(["gzip", filename]) != 0:
                    raise CleanupError, "Error: gzip not returned 0 for file %s." % filename
                # make symlink with original file name
                if subprocess.call(["ln", "-s", os.path.basename(filename) + ".gz",
                                    filename]) != 0:
                    raise CleanupError, "Error: ln not returned 0 for file %s." % filename
            except OSError:
                raise CleanupError, "Error: gzip or ln not on machine."


def cleanup(args, latest_test):
    '''scans given directory for unnecessary files and removes them

    @param args Value instance of parsed arguments
    @param latest_test number of last cleaned test
    @return number of last cleaned test
    '''

    testsdir = args.directory

    dirs_to_check = os.listdir(testsdir)

    # get all directories in format "XY-test"
    dirs_to_check = [drc for drc in dirs_to_check if re.match(r"^\d+-.+$", drc)]

    # sort directories by test number
    def sort_num(x, y):
        numx = int(x[:x.find('-')])
        numy = int(y[:y.find('-')])
        return cmp(numx, numy)

    dirs_to_check.sort(sort_num)

    # don't check old logs (logs that were already checked)
    if latest_test:
        dirs_to_check = filter(lambda x: int(x[:x.find("-")]) > latest_test, dirs_to_check)
    else:
        latest_test = -1

    # iterate over all directories that should be checked
    for directory in dirs_to_check:

        dirpath = os.path.join(testsdir, directory)
        # find test number
        testpos = int(directory[:directory.find("-")])

        try:

            dirs = get_dirs(dirpath)

            # iterate over each folder in directory (one folder for one machine)
            for machinepath, machinedir in dirs:
                try:

                    testname = get_test_name(machinepath)

                    # create list of files to remove
                    useless_files = USELESS_FILES_TEMPLATE[:]
                    useless_files = [re.sub(r"<name>", testname, uf) for uf
                                     in useless_files]

                    # iterate over all folders in each test
                    for path, dirs, files in os.walk(machinepath):

                        clean_files(args, path, useless_files)
                        compress_big_files(args, path)

                    if not args.dry_run:
                        remove_empty_dirs(machinepath)

                except NoTestDirError: # directory doesn't contain test dir
                    if not check_dir_time(machinepath, int(args.age)):
                        logging.debug("Directory %s too new, not clearing, aborting.", machinepath)
                        return latest_test
                    logging.info("Missing 'status' file, passing directory %s.", machinepath)

                except TooYoungError: # there is too new file
                    logging.debug("File in directory %s too new, not clearing, aborting.", machinepath)
                    return latest_test

                except (CleanupError, OSError): # other error during cleaning
                    logging.warn("Error during cleanup, passing directory %s.", machinepath, exc_info=True)

        except OSError: # problem with rights, ownership
            logging.warn("Error during cleanup, passing directory %s.", dirpath, exc_info=True)

        if testpos > latest_test:
            latest_test = testpos

    return latest_test

if __name__ == "__main__":
    logging.basicConfig(filename=LOG_PATH, level=logging.DEBUG, format="%(levelname)-8s %(asctime)-15s  %(message)s")
    args = parse_arguments()
    latest_test = load_last_run()
    latest_test = cleanup(args, latest_test)
    if not args.dry_run:
        save_last_run(latest_test)
